@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * Author:  Roberto Majadas <roberto.majadas@hispalinux.es>
+ * Author:  Roberto Majadas <roberto.majadas@openshine.com>
  */
 
 #include <config.h>
@@ -26,15 +26,22 @@
 #include <dirent.h>
 #include <gnome.h>
 #include <glade/glade.h>
+#include <gconf/gconf-client.h>
 #include "nautilus-sendto-plugin.h"
 
+#define NAUTILUS_SENDTO_GCONF		"/desktop/gnome/nautilus-sendto"
+#define NAUTILUS_SENDTO_LAST_MEDIUM	NAUTILUS_SENDTO_GCONF"/last_medium"
+#define NAUTILUS_SENDTO_LAST_COMPRESS	NAUTILUS_SENDTO_GCONF"/last_compress"
+
 static 
 gchar *default_url = NULL;
 gboolean force_user_to_compress = FALSE;
 GList *file_list = NULL;
 GList *plugin_list = NULL;
 GHashTable *hash ;
-guint option;
+guint option = 0;
+
+static GConfClient *gconf_client = NULL;
 
 typedef struct _NS_ui NS_ui;
 
@@ -63,6 +70,96 @@ destroy_dialog (GtkWidget *widget, gpointer data )
         gtk_main_quit ();
 }
 
+static char *
+get_filename_from_list (void)
+{
+	GList *l;
+	GString *common_part = NULL;
+	gboolean matches = TRUE;
+	guint offset = 0;
+	const char *encoding;
+	gboolean use_utf8 = TRUE;
+
+	encoding = g_getenv ("G_FILENAME_ENCODING");
+
+	if (encoding != NULL && strcasecmp(encoding, "UTF-8") != 0)
+		use_utf8 = FALSE;
+
+	if (file_list == NULL)
+		return NULL;
+
+	common_part = g_string_new("");
+
+	while (TRUE) {
+		gunichar cur_char = '\0';
+		for (l = file_list; l ; l = l->next) {
+			char *path = NULL, *name = NULL;
+			char *offset_name = NULL;
+
+			path = g_filename_from_uri ((char *) l->data,
+					NULL, NULL);
+			if (!path)
+				break;
+
+			name = g_path_get_basename (path);
+
+			if (!use_utf8) {
+				char *tmp;
+
+				tmp = g_filename_to_utf8 (name, -1,
+						NULL, NULL, NULL);
+				g_free (name);
+				name = tmp;
+			}
+
+			if (!name) {
+				g_free (path);
+				break;
+			}
+
+			if (offset >= g_utf8_strlen (name, -1)) {
+				g_free(name);
+				g_free(path);
+				matches = FALSE;
+				break;
+			}
+
+			offset_name = g_utf8_offset_to_pointer (name, offset);
+
+			if (offset_name == g_utf8_strrchr (name, -1, '.')) {
+				g_free (name);
+				g_free (path);
+				matches = FALSE;
+				break;
+			}
+			if (cur_char == '\0') {
+				cur_char = g_utf8_get_char (offset_name);
+			} else if (cur_char != g_utf8_get_char (offset_name)) {
+				g_free (name);
+				g_free (path);
+				matches = FALSE;
+				break;
+			}
+			g_free (name);
+			g_free (path);
+		}
+		if (matches == TRUE && cur_char != '\0') {
+			offset++;
+			common_part = g_string_append_unichar (common_part,
+					cur_char);
+		} else {
+			break;
+		}
+	}
+
+	if (g_utf8_strlen (common_part->str, -1) < 4) {
+		g_string_free (common_part, TRUE);
+		return NULL;
+	}
+
+	return g_string_free (common_part, FALSE);
+}
+
 static char *
 pack_files (NS_ui *ui)
 {
@@ -74,9 +171,9 @@ pack_files (NS_ui *ui)
 	if (strlen (gtk_entry_get_text(GTK_ENTRY(ui->pack_entry))) != 0)
 	{
 		GList *l;
-		GString *cmd, *packed_file;
-		gchar *pack_type, *tmp_dir, *tmp_work_dir;
-		
+		GString *cmd, *tmp;
+		char *pack_type, *tmp_dir, *tmp_work_dir, *packed_file;
+
 		tmp_dir = g_strdup_printf ("%s/nautilus-sendto-%s", 
 				   g_get_tmp_dir(), g_get_user_name());	
 		mkdir (tmp_dir, 0700);
@@ -84,9 +181,9 @@ pack_files (NS_ui *ui)
 						g_get_tmp_dir(), g_get_user_name(),
 						time(NULL));
 		mkdir (tmp_work_dir, 0700);
-		g_free(tmp_dir);
+		g_free (tmp_dir);
 
-		switch (gtk_combo_box_get_active(GTK_COMBO_BOX(ui->pack_combobox)))
+		switch (gtk_combo_box_get_active (GTK_COMBO_BOX(ui->pack_combobox)))
 		{
 		case 0:
 			pack_type = g_strdup (".zip");
@@ -99,10 +196,15 @@ pack_files (NS_ui *ui)
 			break;
 		}
 
+		gconf_client_set_int(gconf_client, 
+				NAUTILUS_SENDTO_LAST_COMPRESS, 
+				gtk_combo_box_get_active(GTK_COMBO_BOX(ui->pack_combobox)), 
+				NULL);
+
 		cmd = g_string_new ("");
 		g_string_printf (cmd, "%s --add-to=\"%s/%s%s\"",
 				 file_roller_cmd, tmp_work_dir,
-				 gtk_entry_get_text(GTK_ENTRY(ui->pack_entry)),
+				 gtk_entry_get_text (GTK_ENTRY(ui->pack_entry)),
 				 pack_type);
 
 		/* file-roller doesn't understand URIs */
@@ -117,12 +219,14 @@ pack_files (NS_ui *ui)
 		g_printf ("%s\n", cmd->str);
 		g_spawn_command_line_sync (cmd->str, NULL, NULL, NULL, NULL);
 		g_string_free (cmd, TRUE);
-		packed_file = g_string_new("");
-		g_string_printf (packed_file,"%s/%s%s",tmp_work_dir,
- 				 gtk_entry_get_text(GTK_ENTRY(ui->pack_entry)),
+		tmp = g_string_new("");
+		g_string_printf (tmp,"%s/%s%s", tmp_work_dir,
+ 				 gtk_entry_get_text (GTK_ENTRY(ui->pack_entry)),
 				 pack_type);
 		g_free (tmp_work_dir);
-		return g_string_free (packed_file, FALSE);
+		packed_file = g_filename_to_uri (tmp->str, NULL, NULL);
+		g_string_free(tmp, TRUE);
+		return packed_file;
 	}else{
 		error_dialog = gtk_message_dialog_new (GTK_WINDOW(ui->dialog),
 						       GTK_DIALOG_MODAL,
@@ -150,14 +254,20 @@ send_button_cb (GtkWidget *widget, gpointer data)
 	
 	if (p == NULL)
 		return;
-		
+
+	gconf_client_set_string (gconf_client, 
+				NAUTILUS_SENDTO_LAST_MEDIUM, p->info->id, NULL);
+
 	if (force_user_to_compress){
 		f = pack_files (ui);
 		if (f != NULL){
 			GList *packed_file = NULL;		
 			packed_file = g_list_append (packed_file, f);
-			if (!p->info->send_files (p, w, packed_file))
+			if (!p->info->send_files (p, w, packed_file)) {
+				g_list_free (packed_file);
 				return;
+			}
+			g_list_free (packed_file);
 		}else{
 			return;
 		}
@@ -167,15 +277,23 @@ send_button_cb (GtkWidget *widget, gpointer data)
 			if (f != NULL){
 				GList *packed_file = NULL;
 				packed_file = g_list_append (packed_file, f);
-				if (!p->info->send_files (p, w, packed_file))
+				if (!p->info->send_files (p, w, packed_file)) {
+					g_list_free (packed_file);
 					return;
+				}
+				g_list_free (packed_file);
 			}else{
 				return;
 			}
 				
 		}else{
-			if (!p->info->send_files (p, w, file_list))
+			if (!p->info->send_files (p, w, file_list)) {
+				g_list_free (file_list);
+				file_list = NULL;
 				return;
+			}
+			g_list_free (file_list);
+			file_list = NULL;
 		}
 	}
 	destroy_dialog (NULL,NULL);
@@ -253,10 +371,16 @@ set_model_for_options_combobox (NS_ui *ui){
 	GtkCellRenderer *renderer;
 	GList *aux;
 	NstPlugin *p;
-	
+	gchar *last_used = NULL;
+	int i = 0;
+
 	it = gtk_icon_theme_get_default ();
 
 	model = gtk_tree_store_new (2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
+
+	last_used = gconf_client_get_string (gconf_client,
+			NAUTILUS_SENDTO_LAST_MEDIUM, NULL);
+
 	for (aux = plugin_list; aux; aux = aux->next){
 		p = (NstPlugin *) aux->data;
 		pixbuf = gtk_icon_theme_load_icon (it, p->info->icon, 16, 
@@ -266,7 +390,12 @@ set_model_for_options_combobox (NS_ui *ui){
 					0, pixbuf,
 					1, p->info->description,
 					-1);
+		if (last_used != NULL && !strcmp(last_used, p->info->id))
+			option = i;
+		i++;
 	}
+	g_free(last_used);
+
 	gtk_combo_box_set_model (GTK_COMBO_BOX(ui->options_combobox),
 				GTK_TREE_MODEL (model));
 	renderer = gtk_cell_renderer_pixbuf_new ();
@@ -285,8 +414,8 @@ set_model_for_options_combobox (NS_ui *ui){
 					renderer,
                                         "text", 1,
                                         NULL);
-	gtk_combo_box_set_active (GTK_COMBO_BOX (ui->options_combobox), 0);
-	option = 0;
+
+	gtk_combo_box_set_active (GTK_COMBO_BOX (ui->options_combobox), option);
 }
 
 static void
@@ -295,6 +424,7 @@ nautilus_sendto_create_ui (void)
 	GladeXML *app;	
 	gint toggle;
 	NS_ui *ui;
+	gboolean one_file = FALSE;
 
 	if (force_user_to_compress == FALSE)
 		app = glade_xml_new (GLADEDIR "/" "nautilus-sendto.glade", NULL, NULL);
@@ -312,8 +442,39 @@ nautilus_sendto_create_ui (void)
 	ui->pack_entry = glade_xml_get_widget (app, "pack_entry");
 	
 		
-	gtk_combo_box_set_active (GTK_COMBO_BOX(ui->pack_combobox), 0);
+ 	gtk_combo_box_set_active (GTK_COMBO_BOX(ui->pack_combobox), 
+ 		gconf_client_get_int(gconf_client, 
+ 				NAUTILUS_SENDTO_LAST_COMPRESS, NULL));
+ 
+
+	if (file_list != NULL && file_list->next != NULL)
+		one_file = FALSE;
+	else if (file_list != NULL)
+		one_file = TRUE;
+	
 	gtk_entry_set_text (GTK_ENTRY (ui->pack_entry), _("Files"));
+
+	if (one_file) {
+		char *filepath = NULL, *filename = NULL;
+
+		filepath = g_filename_from_uri ((gchar *)file_list->data,
+				NULL, NULL);
+
+		if (filepath != NULL)
+			filename = g_path_get_basename (filepath);
+		if (filename != NULL && filename[0] != '\0')
+			gtk_entry_set_text (GTK_ENTRY (ui->pack_entry), filename);
+
+		g_free (filename);
+		g_free (filepath);
+	} else {
+		char *filename = get_filename_from_list ();
+		if (filename != NULL && filename[0] != '\0') {
+			gtk_entry_set_text (GTK_ENTRY (ui->pack_entry),
+					filename);
+		}
+		g_free (filename);
+	}
 /*	create_entry_completion (ui->entry); */
 	set_contact_widgets (ui);
 	set_model_for_options_combobox (ui);
@@ -505,11 +666,14 @@ int main (int argc, char **argv)
 				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Nautilus Sendto"),
 				      NULL);
 
+	gconf_client = gconf_client_get_default();
 	nautilus_sendto_init (program, argc, argv);	
 	nautilus_sendto_plugin_init ();
 	nautilus_sendto_create_ui();
 			
 	gtk_main ();
+	g_object_unref(gconf_client);
+
 	return 0;
 }
 
